-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftSpellIcon")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Spell Icon"] = true,
	["Spell icon placement options"] = true,
	["Enable"] = true,
	["Enable spell icon display on the nameplate"] = true,
	["Position"] = true,
	["Adjust spell icon position"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the spell icon"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the cast bar to anchor the spell icon"] = true,
	["Size"] = true,
	["Size in pixels of the spell icon"] = true,
	["X Offset"] = true,
	["X offset of the spell icon"] = true,
	["Y Offset"] = true,
	["Y offset of the spell icon"] = true,
} end)

-----------------------------------------------------------------------------

local AloftSpellIcon = Aloft:NewModule("SpellIcon", "AceEvent-2.0")
AloftSpellIcon.db = Aloft:AcquireDBNamespace("spellIcon")
Aloft:RegisterDefaults("spellIcon", "profile", {
	enable			= true,
	point			= "RIGHT",
	relativeToPoint	= "LEFT",
	offsetX			= 0,
	offsetY			= 0,
	size			= 12
})

Aloft.Options.args.castBar.args.spellIcon =
{
	type = 'group',
	name = L["Spell Icon"],
	desc = L["Spell icon placement options"],
	disabled = function() return not Aloft:IsActive() or GetCVar("ShowVKeyCastbar") ~= "1" end,
	order = -1,
	args =
	{
		enable = 
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable spell icon display on the nameplate"],
			get = function() return AloftSpellIcon.db.profile.enable end,
			set = function(v) AloftSpellIcon.db.profile.enable = v AloftSpellIcon:UpdateAll() end,
			order = 1
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust spell icon position"],
			disabled = function() return not AloftSpellIcon.db.profile.enable end,
			args =
			{
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the spell icon"],
					get = function() return AloftSpellIcon.db.profile.point end,
					set = function(v) AloftSpellIcon.db.profile.point = v AloftSpellIcon:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the cast bar to anchor the spell icon"],
					get = function() return AloftSpellIcon.db.profile.relativeToPoint end,
					set = function(v) AloftSpellIcon.db.profile.relativeToPoint = v AloftSpellIcon:UpdateAll() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the spell icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftSpellIcon.db.profile.offsetX end,
					set = function(v) AloftSpellIcon.db.profile.offsetX = v AloftSpellIcon:UpdateAll() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the spell icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftSpellIcon.db.profile.offsetY end,
					set = function(v) AloftSpellIcon.db.profile.offsetY = v AloftSpellIcon:UpdateAll() end
				},
			},
		},
		size =
		{
			type = 'range',
			name = L["Size"],
			desc = L["Size in pixels of the spell icon"],
			min = 4,
			max = 32,
			step = 1,
			disabled = function() return not AloftSpellIcon.db.profile.enable end,
			get = function() return AloftSpellIcon.db.profile.size end,
			set = function(v) AloftSpellIcon.db.profile.size = v AloftSpellIcon:UpdateAll() end
		},
	},
}

-----------------------------------------------------------------------------

function AloftSpellIcon:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		if aloftData:IsCurrentTarget() then
			self:OnCastBarShow(aloftData)
		end
	end
end

-----------------------------------------------------------------------------

function AloftSpellIcon:OnEnable()
	self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
	self:UpdateAll()
end

function AloftSpellIcon:OnDisable()
	self:UnregisterAllEvents()
	
	for aloftData in Aloft:IterateNameplates() do
		local spellIconRegion = aloftData.spellIconRegion
		spellIconRegion:ClearAllPoints()
		spellIconRegion:SetPoint("CENTER", aloftData.nameplateFrame, "BOTTOMLEFT", 13.33, 10.29)
		spellIconRegion:SetWidth(14.5)
		spellIconRegion:SetHeight(14.5)
	end
end

function AloftSpellIcon:OnCastBarShow(aloftData)
	if self.db.profile.enable then
		self:PlaceFrame(aloftData.spellIconRegion, aloftData.castBar, self.db.profile)
	else
		aloftData.spellIconRegion:SetTexture("")
		aloftData.spellIconRegion:SetAlpha(0)
		aloftData.spellIconRegion:Hide()
	end
end

-----------------------------------------------------------------------------
