-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftShortGuildData")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Reset Short Guilds"] = true,
	["Empties the custom short guild database"] = true,

	["ShortSubstitutions"] =
	{
		[" [Aa][Nn][Dd] "] = " & ",
		[" [Oo][Ff] "] = " of ",		-- Forces shortened form "o" to go lower case
		[" [Tt][Hh][Ee] "] = " the ",
		[" [Ii][Nn] "] = " in ",
	}
} end)

-----------------------------------------------------------------------------

local shortGuildList = { }
local shortGuildCache = { }

-----------------------------------------------------------------------------

local AloftShortGuildData = Aloft:NewModule("ShortGuildData", "AceEvent-2.0")
AloftShortGuildData.db = Aloft:AcquireDBNamespace("shortGuildData")
Aloft:RegisterDefaults("shortGuildData", "server", { shortGuildList = { } })

-----------------------------------------------------------------------------

Aloft.Options.args.data.args.shortGuild =
{
    type = "execute",
    name = L["Reset Short Guilds"],
    desc = L["Empties the custom short guild database"],
    func = function() AloftShortGuildData:ClearAllData() end,
}

-----------------------------------------------------------------------------

AloftTagDirectAccessData.shortGuild = true
AloftTagData.shortGuild = { data = "shortGuild", events = "Aloft:OnGuildDataChanged" }

function AloftShortGuildData:ProvidesData()
	return "shortGuild"
end

function AloftShortGuildData:RequiresData()
	return "guild"
end

function AloftShortGuildData:EnableDataSource()
end

-----------------------------------------------------------------------------

function AloftShortGuildData:OnEnable()
	shortGuildList = self.db.server.shortGuildList
end

-----------------------------------------------------------------------------

local updateRequired = false

local function AddData(name, guild)
	if name and guild and playerGuilds[name] ~= guild then
		playerGuilds[name] = guild
		updateRequired = true
	end
end

function AloftShortGuildData:UpdateVisibleNameplates()
	if updateRequired then
		for aloftData in Aloft:IterateVisibleNameplates() do
			local guild = playerGuilds[aloftData.name]
		
			if aloftData.guild ~= guild then
				Aloft:UpdateData(aloftData)
				self:TriggerEvent("Aloft:OnGuildDataChanged", aloftData)
			end
		end
	end
	updateRequired = false
end

function AloftShortGuildData:UpdateAll()
	for aloftData in Aloft:IterateVisibleNameplates() do
		local guild = aloftData.guild

		if guild and aloftData.shortGuild ~= (shortGuildList[guild] or shortGuildCache[guild]) then
			Aloft:UpdateData(aloftData)
			self:TriggerEvent("Aloft:OnGuildDataChanged", aloftData)
		end
	end
end

function AloftShortGuildData:AddData(guild, abbreviation)
	shortGuildList[guild] = abbreviation
	self:UpdateAll()
end

function AloftShortGuildData:ClearAllData()
	for k in pairs(shortGuildList) do
		shortGuildList[k] = nil
	end
	self:UpdateAll()
end

function AloftShortGuildData:CreateShortName(guild)
	local _, numberOfSpaces = guild:gsub(" ", "")
	
	if numberOfSpaces == 0 then
		return guild:sub(1, 4)
	end

	-- This code replaces 2-word guild names with the first two characters of each word
	-- In practise I preferred not having the extra chars, than having the more obvious distinction
--	if numberOfSpaces == 1 then
--		return guild:gsub("(%a%a?)%a*%A*", "%1")
--	end

	for k,v in pairs(L["ShortSubstitutions"]) do
		guild = guild:gsub(k, v)
	end

	return guild:gsub("(%a)%a*%A*", "%1")
end

-----------------------------------------------------------------------------

function AloftShortGuildData:UpdateData(aloftData)
	aloftData.shortGuild = nil
	
	local guild = aloftData.guild	
	if guild then
		local shortGuild = shortGuildList[guild] or shortGuildCache[guild]
		
		if not shortGuild then
			shortGuild = self:CreateShortName(guild)
			shortGuildCache[guild] = shortGuild
		end
		
		aloftData.shortGuild = shortGuild
	end
end

-----------------------------------------------------------------------------
