-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftVisibility")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Visibility"] = true,
	["Extra visibility options"] = true,

	["Show Hostile Players"] = true,
	["Shows nameplates for hostile players"] = true,
	["Show Hostile Pets"] = true,
	["Shows nameplates for hostile pets"] = true,
	["Show Hostile NPCs"] = true,
	["Show hostile NPCs"] = true,

	["Friendly Players"] = true,
	["Selects which friendly players to show"] = true,
	["Show Pet"] = true,
	["Shows nameplates for your pets"] = true,
	["Show Group Pets"] = true,
	["Shows nameplates for group pets"] = true,
	["Show Friendly Pets"] = true,
	["Shows nameplates for friendly pets"] = true,
	["Show Friendly NPCs"] = true,
	["Show friendly NPCs"] = true,
	
	["Show Vermin"] = true,
	["Shows nameplates for level 1 neutral units"] = true,
	["Show Neutral Units"] = true,
	["Show neutral units"] = true,

	["All"] = true,
	["Group Only"] = true,
	["Guild Only"] = true,
	["None"] = true,
} end)

-----------------------------------------------------------------------------

local AloftVisibility = Aloft:NewModule("Visibility", "AceEvent-2.0")
AloftVisibility.db = Aloft:AcquireDBNamespace("Visibility")
Aloft:RegisterDefaults("Visibility", "profile", {
	showHostilePlayers = true,
	showHostilePets = true,
	showHostileNPCs = true,

	friendlyPlayers = "ALL",
	showPet = true,
	showGroupPets = true,
	showFriendlyPets = true,
	showFriendlyNPCs = true,

	showVermin = true,
	showNeutralUnits = true,
})

Aloft.Options.args.visibility =
{
	type = 'group',
	name = L["Visibility"],
	desc = L["Extra visibility options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		friendlyPlayers =
		{
			type = 'text',
			name = L["Friendly Players"],
			desc = L["Selects which friendly players to show"],
			get = function() return AloftVisibility.db.profile.friendlyPlayers end,
			set = function(v) AloftVisibility.db.profile.friendlyPlayers = v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			validate = { ["ALL"] = L["All"], ["GUILDONLY"] = L["Guild Only"], ["GROUPONLY"] = L["Group Only"], ["NONE"] = L["None"] },
			order = 1,
		},
		showPet =
		{
			type = 'toggle',
			name = L["Show Pet"],
			desc = L["Shows nameplates for your pets"],
			get = function() return AloftVisibility.db.profile.showPet end,
			set = function(v) AloftVisibility.db.profile.showPet = v AloftVisibility:Update() end,
			order = 2,
		},
		showGroupPets =
		{
			type = 'toggle',
			name = L["Show Group Pets"],
			desc = L["Shows nameplates for group pets"],
			get = function() return AloftVisibility.db.profile.showGroupPets end,
			set = function(v) AloftVisibility.db.profile.showGroupPets = v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			order = 3,
		},
		showFriendlyPets =
		{
			type = 'toggle',
			name = L["Show Friendly Pets"],
			desc = L["Shows nameplates for friendly pets"],
			get = function() return AloftVisibility.db.profile.showFriendlyPets end,
			set = function(v) AloftVisibility.db.profile.showFriendlyPets = v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			order = 4,
		},
		showFriendlyNPCs =
		{
			type = 'toggle',
			name = L["Show Friendly NPCs"],
			desc = L["Show friendly NPCs"],
			get = function() return AloftVisibility.db.profile.showFriendlyNPCs end,
			set = function(v) AloftVisibility.db.profile.showFriendlyNPCs= v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			order = 5,
		},
		showHostilePlayers =
		{
			type = 'toggle',
			name = L["Show Hostile Players"],
			desc = L["Shows nameplates for hostile players"],
			get = function() return AloftVisibility.db.profile.showHostilePlayers end,
			set = function(v) AloftVisibility.db.profile.showHostilePlayers = v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			order = 6,
		},
		showHostilePets =
		{
			type = 'toggle',
			name = L["Show Hostile Pets"],
			desc = L["Shows nameplates for hostile pets"],
			get = function() return AloftVisibility.db.profile.showHostilePets  end,
			set = function(v) AloftVisibility.db.profile.showHostilePets = v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			order = 7,
		},
		showHostileNPCs =
		{
			type = 'toggle',
			name = L["Show Hostile NPCs"],
			desc = L["Show hostile NPCs"],
			get = function() return AloftVisibility.db.profile.showHostileNPCs end,
			set = function(v) AloftVisibility.db.profile.showHostileNPCs = v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			order = 8,
		},
		showNeutralUnits =
		{
			type = 'toggle',
			name = L["Show Neutral Units"],
			desc = L["Show neutral units"],
			get = function() return AloftVisibility.db.profile.showNeutralUnits end,
			set = function(v) AloftVisibility.db.profile.showNeutralUnits = v AloftVisibility:Update() end,
			order = 9,
		},
		showVermin =
		{
			type = 'toggle',
			name = L["Show Vermin"],
			desc = L["Shows nameplates for level 1 neutral units"],
			disabled = function() return not AloftVisibility.db.profile.showNeutralUnits end,
			get = function() return AloftVisibility.db.profile.showVermin end,
			set = function(v) AloftVisibility.db.profile.showVermin = v AloftVisibility:Update() end,
			order = 10
		},
	},
}

-----------------------------------------------------------------------------

function AloftVisibility:Update()
	self:RegisterEvents()

	-- Toggle the visibility to get previously hidden stuff to show
	Aloft:RedisplayNameplates()
	Aloft:RedisplayFriendlyNameplates()
end

function AloftVisibility:RegisterEvents()
	self:UnregisterAllEvents()

	if not self.db.profile.showHostilePlayers then
		self:RegisterEvent("Aloft:OnClassDataChanged", "OnClassDataChanged")
	end
	
	if not self.db.profile.showHostilePets or
	   not self.db.profile.showFriendlyPets then
		self:RegisterEvent("Aloft:OnIsPetDataChanged", "OnNameplateShow")
	end

	if not self.db.profile.showVermin or 
	   not self.db.profile.showHostilePlayers or
	   not self.db.profile.showHostilePets or
	   not self.db.profile.showNeutralUnits or
	   not self.db.profile.showFriendlyNPCs or
	   not self.db.profile.showFriendlyPets or
	   not self.db.profile.showGroupPets or
	   not self.db.profile.showPet or
	   self.db.profile.friendlyPlayers ~= "ALL" then
		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	end
end

-----------------------------------------------------------------------------

function AloftVisibility:OnEnable()	
	self:RegisterEvents()
end

function AloftVisibility:OnDisable()	
	self:UnregisterAllEvents()

	if not self.db.profile.showVermin or 
	   not self.db.profile.showHostilePlayers or
	   not self.db.profile.showHostilePets or
	   not self.db.profile.showNeutralUnits or
	   not self.db.profile.showFriendlyNPCs or
	   not self.db.profile.showFriendlyPets  or
	   not self.db.profile.showGroupPets or
	   not self.db.profile.showPet or
	   self.db.profile.friendlyPlayers ~= "ALL" then
		Aloft:RedisplayNameplates()
		Aloft:RedisplayFriendlyNameplates()
	end
end

local dataRequiredList = { }
function AloftVisibility:RequiresData()
	for i = 1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	if not self.db.profile.showHostilePlayers then
		table.insert(dataRequiredList, "class")
	end

	if self.db.profile.showHostilePets ~= self.db.profile.showHostileNPCs or 
	   self.db.profile.showFriendlyPets ~= self.db.profile.showFriendlyNPCs then
		table.insert(dataRequiredList, "isPet")
	end

	if self.db.profile.friendlyPlayers == "GUILDONLY" then
		table.insert(dataRequiredList, "isGuildMember")
	end

	if Aloft:IsDataAvailable("unitid") then
		if self.db.profile.friendlyPlayers == "GROUPONLY" or
	   	   self.db.profile.showGroupPets ~= self.db.profile.showFriendlyPets then
			table.insert(dataRequiredList, "unitid")
		end
	end

	return unpack(dataRequiredList)
end

-----------------------------------------------------------------------------

local friendlyPlayerMethods =
{
	["ALL"] = function() end,
	["GUILDONLY"] = function(frame, aloftData) if not aloftData.isGuildMember then frame:Hide() end end,
	["GROUPONLY"] = function(frame, aloftData) if not aloftData.unitid then frame:Hide() end end,
	["NONE"] = function(frame) frame:Hide() end,
}

local showTypeMethods =
{
	["friendlyPlayer"] = function(frame, aloftData) 
							local friendlyPlayerMethod = friendlyPlayerMethods[AloftVisibility.db.profile.friendlyPlayers]
							if friendlyPlayerMethod then
								friendlyPlayerMethod(frame, aloftData)
							end
						 end,
	["friendlyNPC"] = function(frame, aloftData) 
						if UnitName("pet") == aloftData.name then
							if not AloftVisibility.db.profile.showPet then
								frame:Hide()
							end
						elseif aloftData.unitid then
							-- Group pet
							if not AloftVisibility.db.profile.showGroupPets then
								frame:Hide()
							end
						elseif aloftData.isPet then
							if not AloftVisibility.db.profile.showFriendlyPets then
								frame:Hide()
							end
						else
							if not AloftVisibility.db.profile.showFriendlyNPCs then 
								frame:Hide() 
							end
						end
					  end,
	["neutral"] = function(frame, aloftData) 
					if not AloftVisibility.db.profile.showNeutralUnits or (not AloftVisibility.db.profile.showVermin and aloftData.level == 1) then 
						frame:Hide()
					end
				  end,
	["hostile"] = function(frame, aloftData)
					-- Is it a player?
					if aloftData.class then 
						if not AloftVisibility.db.profile.showHostilePlayers then
							frame:Hide()
						end
					elseif aloftData.isPet then
						if not AloftVisibility.db.profile.showHostilePets then
							frame:Hide()
						end
					else
						if not AloftVisibility.db.profile.showHostileNPCs then
							frame:Hide()
						end
					end
				  end,
	["unknown"] = function() end,
}

function AloftVisibility:OnNameplateShow(aloftData)
	showTypeMethods[aloftData.type](aloftData.nameplateFrame, aloftData)
end

function AloftVisibility:OnClassDataChanged(aloftData)
	-- If it was changed, then it must have some value now
	if aloftData.type == "hostile" then
		aloftData.nameplateFrame:Hide()
	end
end

-----------------------------------------------------------------------------
